#!/bin/bash
# Author: josejp2424
# Description: A graphical tool to manage GRUB. It automatically detects the bootable disk and partition for GRUB installation and generates GRUB entries for .sfs files.
# License: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
export TEXTDOMAIN=grub_manager
export OUTPUT_CHARSET=UTF-8

# Internationalized text variables
mytitle=$(gettext 'Please wait...')
mytitle2=$(gettext 'GRUB Manager')
mytitle3=$(gettext 'Update')
mytitle4=$(gettext 'Install')
mytitle5=$(gettext 'Exit')
mytitle6=$(gettext 'Updating GRUB')
mytitle7=$(gettext 'Updating GRUB configuration...')
mytitle8=$(gettext 'Cancel')
mytitle9=$(gettext 'GRUB updated successfully.')
mytitle10=$(gettext 'GRUB installed successfully.')
mytitle11=$(gettext 'Exiting...')
mytitle12=$(gettext 'Installing GRUB')
mytitle13=$(gettext 'Installing GRUB on disk...')
mytitle14=$(gettext 'Choose an option to manage GRUB:')
mytitle15=$(gettext 'No distributions were detected on the partitions.')
mytitle16=$(gettext 'Entries were generated')
mytitle17=$(gettext 'GRUB entries in')
mytitle18=$(gettext 'The file')
mytitle19=$(gettext 'has been modified successfully.')
mytitle20=$(gettext 'Unable to detect bootable disk or partition. Ensure a boot partition is properly mounted.')

ICON_PATH="/usr/local/grub_manager/grub-customizer.svg"
GRUB_CUSTOM="/etc/grub.d/40_custom"
mount | grep overlay

# Function to generate GRUB entries from .sfs files
generate_partition_entries() {
    local grub_custom="$1"
    local entry_count=0

    # Create a backup of the original file (if it doesn't already exist)
    [[ ! -f "${grub_custom}.bak" ]] && cp "$grub_custom" "${grub_custom}.bak"

    # Keep existing header of 40_custom
    head -n 10 "${grub_custom}.bak" > "$grub_custom"

    # Loop through partitions in /dev/sd*
    for partition in /dev/sd*[0-9]; do
        # Extract UUID from the partition
        uuid=$(blkid -o value -s UUID "$partition" 2>/dev/null)
        [[ -z "$uuid" ]] && continue

        # Create a temporary mount point
        mount_dir=$(mktemp -d)

        # Mount the partition
        mount "$partition" "$mount_dir" 2>/dev/null || continue

        # Search for .sfs files that meet the criteria
        find "$mount_dir" -type f -name "*.sfs" | while read -r sfs_file; do
            base_dir=$(dirname "$sfs_file")
            base_name=$(basename "$base_dir")
            kernel_file="$base_dir/vmlinuz"
            initrd_file="$base_dir/initrd.gz"

            # Check that the file name meets the criteria
            if [[ -f "$kernel_file" && -f "$initrd_file" ]]; then
                # Create a GRUB entry based on the name of the .sfs file
                if [[ "$(basename "$sfs_file")" =~ ^puppy.*\.sfs$ ]]; then
                    # Configuration for Puppy
                    distro_name=$(basename "$sfs_file" | sed 's/.sfs$//')
                    echo "menuentry '$distro_name (${partition}/${base_name})' --class gnu-linux{" >> "$grub_custom"
                    echo "    search --no-floppy --set=root --fs-uuid $uuid" >> "$grub_custom"
                    echo "    linux /${base_name}/vmlinuz psubdir=/${base_name} pmedia=atahd pfix=fsck" >> "$grub_custom"
                    echo "    initrd /${base_name}/initrd.gz" >> "$grub_custom"
                    echo "}" >> "$grub_custom"
                    echo "" >> "$grub_custom"
                elif [[ "$(basename "$sfs_file")" =~ ^07.*\.sfs$ ]]; then
                    # Configuration for files starting with 07
                    distro_name=$(basename "$sfs_file" | sed 's/.sfs$//')
                    echo "menuentry '$distro_name (${partition}/${base_name})' --class gnu-linux{" >> "$grub_custom"
                    echo "    search --no-floppy --set=root --fs-uuid $uuid" >> "$grub_custom"
                    echo "    linux /${base_name}/vmlinuz w_bootfrom=UUID=$uuid=/${base_name} w_changes=RAM2" >> "$grub_custom"
                    echo "    initrd /${base_name}/initrd.gz" >> "$grub_custom"
                    echo "}" >> "$grub_custom"
                    echo "" >> "$grub_custom"
                fi

                ((entry_count++))
            fi
        done

        # Unmount the partition
        umount "$mount_dir"
        rmdir "$mount_dir"
    done

    if [[ $entry_count -eq 0 ]]; then
        echo "$mytitle15"
    else
        echo "$mytitle16 $entry_count $mytitle17 $grub_custom."
    fi
}

# Progress window while running the initialization script
(
  yad --title="$mytitle" \
      --text="$mytitle" \
      --window-icon="$ICON_PATH" \
      --image="$ICON_PATH" --image-on-top \
      --center \
      --progress= 40 \
      --auto-close \
      --width=300 \
      --height=100 \
      --no-buttons &

  WAIT_PID=$!  

  # Generate GRUB entries from .sfs files
  generate_partition_entries "$GRUB_CUSTOM"

# Change permissions of the file
chmod +x "$GRUB_CUSTOM"
echo "$mytitle18 $GRUB_CUSTOM $mytitle19"
  kill $WAIT_PID
) &

wait  

# Main window with buttons for options
CHOICE=$(yad --title="$mytitle2" \
  --window-icon="$ICON_PATH" \
  --image="$ICON_PATH" --image-on-top \
  --width=300 --height=150 \
  --center \
  --buttons-layout=horizontal \
  --button="$mytitle3 GRUB:0" \
  --button="$mytitle4 GRUB:1" \
  --button="$mytitle5:2" \
  --text="$mytitle14")

case $? in
  0)
    # Option: Update GRUB
    ( 
      yad --title="$mytitle6" \
          --text="$mytitle7" \
          --window-icon="$ICON_PATH" \
          --image="$ICON_PATH" --image-on-top \
          --center \
          --progress= 80 \
          --auto-close \
          --width=300 \
          --height=100 \
          --no-buttons \
          --timeout=0 &
      GRUB_PID=$!  

      # Check if the system is UEFI
      if [ -d /sys/firmware/efi ]; then
        # UEFI system: Update GRUB for UEFI
        sudo grub-mkconfig -o /boot/efi/EFI/grub/grub.cfg
      else
        # BIOS system: Update GRUB for BIOS
        sudo grub-mkconfig -o /boot/grub/grub.cfg
      fi
      kill $GRUB_PID  

    ) &
    wait  
    yad --window-icon="$ICON_PATH" --center --info --text="$mytitle9" --width=300 --height=100
    ;;
  1)
    # Option: Install GRUB
    (
      yad --title="$mytitle12" \
          --text="$mytitle13\n(Disk: /dev/$DISK, Partition: /dev/$BOOT_PART)" \
          --window-icon="$ICON_PATH" \
          --image="$ICON_PATH" --image-on-top \
          --center \
          --progress= 90 \
          --auto-close \
          --width=300 \
          --height=100 \
          --no-buttons \
          --timeout=0 &
      GRUB_PID=$!  

      # Check if the system is UEFI
      if [ -d /sys/firmware/efi ]; then
        # UEFI system: Install GRUB for UEFI
        sudo grub-install --target=x86_64-efi --efi-directory=/boot/efi --bootloader-id=GRUB
      else
        # BIOS system: Install GRUB for BIOS
        sudo grub-install /dev/$DISK
      fi
      kill $GRUB_PID  
    ) &
    wait  
    yad --window-icon="$ICON_PATH" --center --info --text="$mytitle10" --width=300 --height=100
    ;;
  2)
    # Option: Exit
    yad --window-icon="$ICON_PATH" --center --info --text="$mytitle11" --width=300 --height=100
    exit 0
    ;;
  *)
    exit 1
    ;;
esac
